@@ -11,7 +11,7 @@ import binascii
 import random
 from datetime import datetime,timezone
 import base64
-import platform # platform.node() -> hostname
+import platform  # platform.node() -> hostname
 # for yaml
 #from ruamel.yaml.comments import CommentedMap
 import ruamel.yaml
@@ -45,19 +45,94 @@ c={
 	"important":"\033[1;33m",   # light red
 }
 
+ansi256color={
+	"black":"\x1b[38;5;235m",
+	"gray":"\x1b[38;5;240m",
+	"white":"\x1b[38;5;247m",
+	"lightwhite":"\x1b[38;5;231m",
+	"red":"\x1b[38;5;124m",
+	"lightred":"\x1b[38;5;196m",
+	"green":"\x1b[38;5;28m",
+	"lightgreen":"\x1b[38;5;83m",
+	"maroon":"\x1b[38;5;94m",
+	"yellow":"\x1b[38;5;226m",
+	"blue":"\x1b[38;5;25m",
+	"lightblue":"\x1b[38;5;39m",
+	"pink":"\x1b[38;5;128m",
+	"lightpink":"\x1b[38;5;206m",
+	"cyan":"\x1b[38;5;30m",
+	"lightcyan":"\x1b[38;5;50m",
+	"orange":"\x1b[38;5;166m",
+	"lightorange":"\x1b[38;5;214m",
+	"salmon":"\x1b[38;5;174m",
+	"olive":"\x1b[38;5;106m",
+	"purple":"\x1b[38;5;93m",
+	"violet":"\x1b[38;5;105m",
+	"magenta":"\x1b[38;5;199m",
+}
+
+def colprint(d,header=None,key=None,indent="",crop=True,format='tree',fields=None,fieldcolors=None):
+	if format=="tree":
+		colprint_tree(d,header,key,indent,crop)
+		return
+	if format=="column":
+		colprint_column(d,header,fields=fields,fieldcolors=fieldcolors)
+		return
+
+def colprint_column(data,header=None,fields=None,fieldcolors=None):
+	if fields is None:
+		if fieldcolors is not None:
+			fields=fieldcolors.keys()
+
+	sizes={}
+	for var in fields:
+		sizes[var]=len(var)
+
+	for d in data:
+		for var in fields:
+			if d.get(var):
+				varsize=len(str(d[var]))
+				if varsize>sizes[var]:
+					sizes[var]=varsize
+
+	if header=="up":
+		colprint_column_header(fields, sizes)
+
+	for d in data:
+		for var in fields:
+			data=str(d.get(var))
+			if data is None: data="-"
+			print(ansi256color.get(fieldcolors[var]), end="")
+			print(data,end="")
+			for f in range(0,sizes[var]-len(data)+1):
+				print(" ",end="")
+		print("\x1b[0m")
+
+	if header=="down":
+		colprint_column_header(fields, sizes)
+
+def colprint_column_header(fields,sizes):
+		for var in fields:
+			print("\x1b[48;5;237m",end="")
+			print("\x1b[37m",end="")
+			print(var,end="")
+			for f in range(0,sizes[var]-len(var)+1):
+				print(" ",end="")
+		print("\x1b[0m")
+
 #{{{ colprint: Imprime un dict por pantalla con colores según el tipo de datos
-def colprint(d,header=None,key=None,indent="",crop=True):
+def colprint_tree(d,header=None,key=None,indent="",crop=True):
 	indentchars="    "
 	if(header): print(c["header"]+header+c["none"]);indent=indent+indentchars
 
 	# Si es diccionario iteramos
-	if(type(d)==dict or type(d)==collections.OrderedDict): # or type(d)==CommentedMap):
+	if(type(d)==dict or type(d)==collections.OrderedDict):  # or type(d)==CommentedMap):
 	#if(type(d)==dict or type(d)==collections.OrderedDict or type(d)==CommentedMap):
 	#if hasattr(d,'__iter__') and type(d)!=str:
-		if key is not None : print(indent+c["group"]+key+":"+c["none"]); indent+=indentchars
+		if key is not None: print(indent+c["group"]+key+":"+c["none"]); indent+=indentchars
 		if len(d)==0: print(indent+c["null"]+"empty"+c["none"])
 		else:
-			for k in d: colprint(d[k], key=k, indent=indent)
+			for k in d: colprint_tree(d[k], key=k, indent=indent)
 		return
 
 	# Si es lista listamos con [indice]
@@ -91,8 +166,8 @@ def colprint(d,header=None,key=None,indent="",crop=True):
 	if type(d)==list or type(d)==tuple:
 		# Vacia
 		if len(d)==0:
-			if(key!=None):
-				print(indent+c["group"]+key+"[]: "+c["null"]+"empty"+c["none"]);
+			if(key is not None):
+				print(indent+c["group"]+key+"[]: "+c["null"]+"empty"+c["none"])
 			else:
 				print(indent+c["null"]+"empty"+c["none"])
 			return
@@ -100,22 +175,22 @@ def colprint(d,header=None,key=None,indent="",crop=True):
 		# Lista de strings o ints
 		if type(d[0])==str or type(d[0])==int:
 			i=0
-			if key!=None:
+			if key is not None:
 				print(indent+c["group"]+key+":"+c["none"])
 				for k in d:
-					colprint(str(k), key="["+str(i)+"]", indent=indent+indentchars)
+					colprint_tree(str(k), key="["+str(i)+"]", indent=indent+indentchars)
 					i+=1
 			else:
 				for k in d:
-					colprint(str(k), key="["+str(i)+"]", indent=indent)
+					colprint_tree(str(k), key="["+str(i)+"]", indent=indent)
 					i+=1
 			return
 
 		#for k in d:
 		#	#print(indent+c["group"]+key+"["+str(i)+"]:"+c["none"])
-		#	#colprint(k, indent=indent+indentchars)
+		#	#colprint_tree(k, indent=indent+indentchars)
 		#	#print(indent+c["item"]+"["+str(i)+"]: "+c["none"], end='')
-		#	colprint(k, indent=indent+c["item"]+"["+str(i)+"]: ")
+		#	colprint_tree(k, indent=indent+c["item"]+"["+str(i)+"]: ")
 		#	i=i+1
 		#	#print(k);
 
@@ -125,7 +200,7 @@ def colprint(d,header=None,key=None,indent="",crop=True):
 			for k in d:
 				if key is None: key=""
 				print(indent+c["group"]+key+"["+str(i)+"]:"+c["none"])
-				colprint(k, indent=indent+indentchars)
+				colprint_tree(k, indent=indent+indentchars)
 				i=i+1
 			return
 
@@ -135,7 +210,7 @@ def colprint(d,header=None,key=None,indent="",crop=True):
 		#return
 
 	# Mostramos key
-	if(key!=None): print(indent+c["item"]+key+": ", end='')
+	if(key is not None): print(indent+c["item"]+key+": ", end='')
 	else: print(indent, end='')
 
 	# Mostramos valor
@@ -167,7 +242,7 @@ def colprint(d,header=None,key=None,indent="",crop=True):
 #	elif(t==list):
 #		print
 #		for i in d:
-#			colprint(i,header="x",indent=indent)
+#			colprint_tree(i,header="x",indent=indent)
 #			#print(indent+indentchars+c["str"]+i)
 #			#print(d)
 	elif(t==type(None)):
@@ -196,7 +271,7 @@ def replacevarsold(d, varlist, unknownempty=False, emptyremove=False, unknownnul
 			__debugiter=__debugiter+1
 			for i in d[k]:
 				for di in range(0,__debugiter): print("    ", end="")
-				print(f"* item {i}")
+				#print(f"* item {i}")
 				o[k].append(replacevarsold(i, varlist, unknownempty=unknownempty, emptyremove=emptyremove, unknownnull=unknownnull, emptynull=emptynull, __debugiter=__debugiter))
 			continue
 		else:
@@ -226,7 +301,8 @@ def replacevarsold(d, varlist, unknownempty=False, emptyremove=False, unknownnul
 								else: val="null"
 							else:
 								continue
-					if d[k]==f"${sk}$":
+					#if d[k]==f"${sk}$":  # python 3.6
+					if d[k]=="$"+sk+"$":  # python 3.6
 						# si el valor es exacto ("$var$"), reemplazamos por el tipo de objeto de varlist
 						d[k]=val
 					else:
@@ -264,7 +340,8 @@ def replacevars(data, varlist, flags=[], __debugiter=0):
 		for k,v in data.items():
 			if __debug:
 				for di in range(0,__debugiter): print("    ", end="")
-				print(f"\033[33m- Item '{k}' [{type(data[k]).__name__}]\033[0m")
+				#print(f"\033[33m- Item '{k}' [{type(data[k]).__name__}]\033[0m")  # python 3.6
+				print("\033[33m- Item '"+k+"' ["+type(data[k]).__name__+"]\033[0m")  # python 3.6
 			if type(v)==str:
 				r=replacevars_str(v, varlist, __debugiter=__debugiter)
 				add=True
@@ -288,7 +365,8 @@ def replacevars(data, varlist, flags=[], __debugiter=0):
 		for k in data:
 			if __debug:
 				for di in range(0,__debugiter): print("    ", end="")
-				print(f"\033[33m- Item '{k}' [{type(k).__name__}]\033[0m")
+				#print(f"\033[33m- Item '{k}' [{type(k).__name__}]\033[0m")
+				print("\033[33m- Item '"+k+"' ["+type(k).__name__+"]\033[0m")
 			oi=replacevars(k, varlist, flags=flags, __debugiter=__debugiter)
 			o.append(oi)
 		return o
@@ -302,7 +380,8 @@ def replacevars(data, varlist, flags=[], __debugiter=0):
 		for k in data:
 			if __debug:
 				for di in range(0,__debugiter): print("    ", end="")
-				print(f"\033[33m- Item '{k}' [{type(k).__name__}]\033[0m")
+				#print(f"\033[33m- Item '{k}' [{type(k).__name__}]\033[0m")
+				print("\033[33m- Item '"+k+"' ["+type(k).__name__+"]\033[0m")
 			oi=replacevars(k, varlist, flags=flags, __debugiter=__debugiter)
 			o.add(oi)
 		return o
@@ -310,7 +389,8 @@ def replacevars(data, varlist, flags=[], __debugiter=0):
 	# unknown!
 	if __debug:
 		for di in range(0,__debugiter): print("    ", end="")
-		print(f"\033[1;31m¿{type(data).__name__}?\033[0m")
+		#print(f"\033[1;31m¿{type(data).__name__}?\033[0m")
+		print("\033[1;31m¿"+type(data).__name__+"?\033[0m")
 	return data
 
 #	for k in d:
@@ -339,7 +419,8 @@ def replacevars_str(data, varlist, __debugiter=0):
 	#__debug=True
 	if __debug:
 		for di in range(0,__debugiter+1): print("    ", end="")
-		print(f"\033[34mreplacevars2_str: '{data}'\033[0m", end="")
+		#print(f"\033[34mreplacevars2_str: '{data}'\033[0m", end="")
+		print("\033[34mreplacevars2_str: '"+data+"'\033[0m", end="")
 	out=data
 	p=re.compile("\\$([a-zA-Z0-9_]*)\\$")
 	vars=p.findall(data)
@@ -354,7 +435,7 @@ def replacevars_str(data, varlist, __debugiter=0):
 		# Hay variables
 		for sk in vars:
 			if __debug:
-				print(f" (repl {sk})", end="")
+				print(" (repl "+sk+")", end="")
 			if sk not in varlist: continue
 			val=varlist.get(sk)
 			#if val is None:
@@ -367,7 +448,7 @@ def replacevars_str(data, varlist, __debugiter=0):
 			#			continue
 
 			# TODO: no borrar variables que no existen!
-			if data==f"${sk}$":
+			if data=="$"+sk+"$":
 				# si el valor es exacto ("$var$"), reemplazamos por el tipo de objeto de varlist
 				out=val
 			else:
@@ -377,7 +458,8 @@ def replacevars_str(data, varlist, __debugiter=0):
 		#if(out=="$null$" and emptynull): out=None
 		#if(out=="$null$"): out=None
 		#if(out=="" and emptyremove): out=None  # ???? TODO
-	if __debug: print(f"\033[34m -> '{out}' [{type(out).__name__}]\033[0m")
+	#if out=="#null#": out=None
+	if __debug: print("\033[34m -> '"+out+"' ["+type(out).__name__+"]\033[0m")
 	return out
 #}}}
 
@@ -433,7 +515,7 @@ def runvars(d, unknownempty=False, emptyremove=None):
 				r_buscacmd=re.compile("\(\((.*)\)\)")
 				vars=set(r_buscacmd.findall(d[k]))
 				# No hay variables
-				if(vars==None):
+				if(vars is None):
 					o[k]=d[k]
 				#	continue
 				else:
@@ -447,7 +529,7 @@ def runvars(d, unknownempty=False, emptyremove=None):
 						ex=run.cmd(pars)
 						#if ex.retcode==0:
 						val=ex.out
-						if(val==None):
+						if(val is None):
 							if(unknownempty): val=""
 							else: continue
 						#d[k]=re.sub("\(\("+sk+"\)\)", val, d[k])
@@ -457,7 +539,7 @@ def runvars(d, unknownempty=False, emptyremove=None):
 				r_buscacmd=re.compile(r"$b$(.*)$")
 				vars=set(r_buscacmd.findall(d[k]))
 				# No hay variables
-				if(vars==None):
+				if(vars is None):
 					o[k]=d[k]
 				else:
 					modified=True
@@ -471,7 +553,7 @@ def runvars(d, unknownempty=False, emptyremove=None):
 				r_buscacmd=re.compile(r"$t$(.*)$")
 				vars=set(r_buscacmd.findall(d[k]))
 				# No hay variables
-				if(vars==None):
+				if(vars is None):
 					o[k]=d[k]
 				else:
 					modified=True
@@ -491,7 +573,7 @@ def runvars(d, unknownempty=False, emptyremove=None):
 # Values of dict contained in another dict
 def contained(d1, d2):
 	for k in d1:
-		if d2.get(k)==None: return False
+		if d2.get(k) is None: return False
 		if(type(d1[k])==dict or type(d1[k])==collections.OrderedDict):
 			if not contained(d1[k], d2[k]): return False
 		else:
@@ -503,7 +585,9 @@ def contained(d1, d2):
 
 # XML -> dict
 def fromxml(xmldata):
-	return xmltodict.parse("<xml>"+xmldata+"</xml>", disable_entities=True)[xml]
+	# TODO: funciona?
+	#return xmltodict.parse("<xml>"+xmldata+"</xml>", disable_entities=True)[xml]
+	return dicttoxml.parse("<xml>"+xmldata+"</xml>", disable_entities=True)["xml"]
 
 #}}}
 
@@ -545,7 +629,7 @@ def json_parser_hook(js):
 			dt=dt.replace(tzinfo=timezone.utc)
 			out[key]=dt
 			continue
-		except: pass
+		except Exception: pass
 		# Hora con timezone con milisegundos
 		try:
 			dt=re.sub("Z$", "UTC",value)
@@ -553,17 +637,17 @@ def json_parser_hook(js):
 			dt=dt.replace(tzinfo=timezone.utc)
 			out[key]=dt
 			continue
-		except: pass
+		except Exception: pass
 		# Hora sin timezone sin milisegundos
 		try:
 			out[key]=datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")
 			continue
-		except: pass
+		except Exception: pass
 		# Hora sin timezone con milisegundos
 		try:
 			out[key]=datetime.strptime(value, "%Y-%m-%dT%H:%M:%S.%f")
 			continue
-		except: pass
+		except Exception: pass
 	return out
 
 # dict -> json
@@ -600,7 +684,7 @@ def readyaml(filename):
 				path=os.path.dirname(filename)
 				f=path+os.path.sep+f
 				if os.path.isfile(f):
-					subdata=read(f)
+					subdata=readyaml(f)
 					data.update(subdata)
 		#data["include"]=None
 		data.pop("yaml-include")
